/*
[Hope no one's upset to find a posting of source-code here. It's a *small*
file. -- T.N. ]

Attached is some ANSI C source code to generate a series of random business 
names for fantasy role playing establishments. The code was written by me, 
from an idea posted to the net recently by Vicki Domansky (educ@unixg.ubc.ca).
A typical set of output is shown below; source code is attached.
 
  The glove and horse Alehouse
  The harpy's shield Tavern
  The merry troll Bar
  The crafty dragon Inn
  The challice and harpy Bar
  The amythest emerald Inn
  The spirits and topaz Tavern
  The crafty hammer Alehouse
  The helm and boar Alehouse
  The golden wolf Inn
  The bent skull Alehouse
  The flying emerald Inn
  The clever harpy Tavern
  The sable crystal Alehouse
  The horse and centaur Inn
  The dancing ruby Inn
  The silken wine Bar
  The flying helm Tavern
  The thirsty wyrm Alehouse

[ source code follows --MG]

------------CUT HERE: BUSINESSNAMES.C--------------*/

/***************************************************
 ***************************************************
 *          Fantasy Business-name Generator        *
 ***************************************************
 ***************************************************/

/*
 *  Coded by Tim North, North_TJ@cc.curtin.edu.au   
 *  From an paper-and-dice based system designed
 *  by "Morgaine" (Vicky Domansky) educ@unixg.ubc.ca
 * 
 *  Version 1.0, June 1992.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "business.h"

#define D(sides) 	(rand() % sides + 1)      /* Dice-rolling macro */
#define MaxWord		20		/* Max letters in a table entry */
#define MaxName		80		/* Max size of generated names  */
#define iterations	20		/* Number of names to generate  */

int
main(void)
{	char 	Name[MaxName];
	int		i;
	
	/* Introduce program and seed the random number generator */
	Introduce();
	
	/* Generate and display a list of names */
	for (i=1; i<iterations; i++) {
		GenerateName(Name);
		printf("  %s\n", Name);
	}
}

void
Introduce(void)
{	
	/* Write out an introduction to the program */
	printf("This program generates a series of random business names\n");
	printf("for fantasy role playing establishments.\n\n");
	printf("Here are %d such names:\n", iterations);

	/* Now seed the random number generator with a "randomish"  */
	/* sort of number. "clock()" just returns an approximation  */
	/* to the processor time.                                   */
	srand( (unsigned int) time(NULL) );
}

void
GenerateName(char Name[])
{	char	Loc[MaxWord], S1[MaxWord], S2[MaxWord];
	
	ChooseLocation(Loc); /* "Inn", "Tavern", etc. */

	switch( D(6) ) {
		case 1:
		case 2:
			GenerateFromTableA(S1);
			GenerateFromTableA(S2);
			sprintf(Name, "The %s and %s %s", S1, S2, Loc);
			break;
		case 3:
			GenerateFromTableB(S1);
			GenerateFromTableC(S2);
			sprintf(Name, "The %s\'s %s %s", S1, S2, Loc);
			break;
		default:
			GenerateFromTableD(S1);
			GenerateFromTableA(S2);
			sprintf(Name, "The %s %s %s", S1, S2, Loc);
			break;
	}
}

void
ChooseLocation(char Loc[])
{	int	 	i, NumLocs;
	char 	*Locations[] = {"Inn", "Inne", "Tavern", "Bar", "Alehouse"};
	
	NumLocs = sizeof(Locations)/sizeof(char *);
	strcpy(Loc, Locations[D(NumLocs)-1]);
}

void
GenerateFromTableA(char S[])
{	static int	new, previous;
	
	/* Prevent the same value being chosen twice in succession */
	do 
		new = D(9);
	while (new == previous);
	previous = new;
	
	switch( new ) {
		case 1: ChooseFromTable( 1, S);	break;
		case 2: ChooseFromTable( 3, S);	break;
		case 3: ChooseFromTable(14, S);	break;
		case 4: ChooseFromTable( 9, S);	break;
		case 5: ChooseFromTable(11, S);	break;
		case 6: ChooseFromTable( 2, S);	break;
		case 7: ChooseFromTable( 4, S);	break;
		case 8: ChooseFromTable( 8, S);	break;
		case 9: ChooseFromTable(10, S);	break;
	}
}

void
GenerateFromTableB(char S[])
{	static int	new, previous;
	
	/* Prevent the same value being chosen twice in succession */
	do 
		new = D(6);
	while (new == previous);
	previous = new;
	
	switch( new ) {
		case 1: ChooseFromTable( 6, S);	break;
		case 2: ChooseFromTable( 8, S);	break;
		case 3: ChooseFromTable(10, S);	break;
		case 4: ChooseFromTable( 7, S);	break;
		case 5: ChooseFromTable( 9, S);	break;
		case 6: ChooseFromTable(11, S);	break;
	}
}

void
GenerateFromTableC(char S[])
{	static int	new, previous;
	
	/* Prevent the same value being chosen twice in succession */
	do 
		new = D(10);
	while (new == previous);
	previous = new;
	
	switch( new ) {
		case  1: ChooseFromTable( 1, S);	break;
		case  2: ChooseFromTable( 3, S);	break;
		case  3: ChooseFromTable(14, S);	break;
		case  4: ChooseFromTable( 9, S);	break;
		case  5: ChooseFromTable(11, S);	break;
		case  6: ChooseFromTable( 2, S);	break;
		case  7: ChooseFromTable( 4, S);	break;
		case  8: ChooseFromTable( 8, S);	break;
		case  9: ChooseFromTable(10, S);	break;
		case 10: ChooseFromTable( 5, S);	break;
	}
}

void
GenerateFromTableD(char S[])
{	static int	new, previous;
	
	/* Prevent the same value being chosen twice in succession */
	do 
		new = D(10);
	while (new == previous);
	previous = new;
	
	switch( new ) {
		case  1: ChooseFromTable(12, S);	break;
		case  2: ChooseFromTable(14, S);	break;
		case  3: ChooseFromTable(16, S);	break;
		case  4: ChooseFromTable(18, S);	break;
		case  5: ChooseFromTable(20, S);	break;
		case  6: ChooseFromTable(13, S);	break;
		case  7: ChooseFromTable(15, S);	break;
		case  8: ChooseFromTable(17, S);	break;
		case  9: ChooseFromTable(19, S);	break;
		case 10: ChooseFromTable(21, S);	break;
	}
}

void
ChooseFromTable(int t, char S[])	
{	int	numElems;

	numElems = atoi(table[t-1][0]);
	strcpy(S, table[t-1][D(numElems)]);
}

/*---cut here---
-------------------------------------------------------------------------------
    _--_|\      | Dept Computer Engineering, Curtin University of Technology
   /      \     | Perth. Western Australia. 6102.    Phone: (+61 9) 351 7908
-->\_.--._/     | Internet: North_TJ@cc.curtin.edu.au
         v      | Bitnet:   North_TJ%cc.curtin.edu.au@cunyvm.bitnet
  TIM NORTH     | UUCP:     uunet!munnari.oz!cc.curtin.edu.au!North_TJ
-------------------------------------------------------------------------------
-- 
 Mitchell Gold     /  middleman of
 sundance@rpi.edu /  rec.games.frp.archives
*/
